#pragma once

#include <visa.h>
#include <string>
#include <sstream>
#include <array>
#include <vector>

// Serial configuration
enum ParityType { NONE, ODD, EVEN, MARK, SPACE };
enum FlowControlType { NO_FLOW_CONTROL, XON_XOFF, RTS_CTS, DTR_DSR };
enum StopBitsType { ONE, ONE5, TWO };

enum DisplayMode {XYZ, MMA, MMH};
enum Units {Vm, mWcm2, Wm2};
enum TriggerMode {FREERUN, INTERNAL, EXTERNAL};
enum TriggerEdge {RISING, FALLING};
enum AddressMode {DYNAMIC, STATIC};

struct ProbeID
{
	std::string model, serial, firmware, linearization;
	bool statusOK;
};

struct InterfaceID
{
	std::string model, serial, firmware;
};

struct ErrorLog
{
	int number, channel;
	std::string error;
};

struct ProbeData
{
	double x, y, z, c;
	bool inRange, freqInRange, statusOK;
};

struct DisplayData
{
	double c1, c2, c3, c4, min, max, avg, minHold, maxHold;
};

struct MMAData
{
	double min, max, avg;
};

struct StaticIP
{
	std::string address, subnetMask;
};

struct Status
{
	bool fault, laser, keyPosition, triggered, webActive, syncError, internalCommError;
};

struct AddressInfo
{
	AddressMode mode;
	std::string address;
};

class FieldProbe
{
public:
	int Initialize(std::string VisaResourceName,
		int BaudRate = 19200,
		FlowControlType FlowControl = NO_FLOW_CONTROL,
		ParityType Parity = NONE,
		int DataBits = 8,
		StopBitsType StopBits = ONE);
	ProbeID QueryProbeID();
	InterfaceID QueryInterfaceID();
	ProbeData ReadData();
	int Close();

private:
	ViSession VISAresource;
	std::string SendCommand(std::string command, bool query = false, unsigned int waitAfterSend = 50, unsigned int BytesToRead = 100);
};

class FieldMonitor
{
public:
	int Initialize(std::string VisaResourceName,
		bool reset = true,
		int BaudRate = 19200,
		FlowControlType FlowControl = NO_FLOW_CONTROL,
		ParityType Parity = NONE,
		int DataBits = 8,
		StopBitsType StopBits = ONE);
	void Reset();
	int Close();
	void LocalMode();
	void ConfigureDisplayMode(DisplayMode mode);
	void ConfigureFrequencyCorrection(bool state);
	void SetFrequency(double frequency);
	void RunMMH();
	void StopMMH();
	void ConfigureChannelState(int channel, bool state);
	void ConfigureSampleRate(int channel, int rate);
	void ConfigureAxes(int channel, bool x, bool y, bool z);
	void ZeroProbe(int channel);
	std::string QueryID();
	DisplayMode QueryDisplayMode();
	bool QueryFrequencyCorrection();
	double QueryFrequency();
	bool Query_MMH_State();
	ErrorLog QueryErrorLog();
	bool QueryChannelState(int channel);
	int QuerySampleRate(int channel);
	std::vector<bool> QueryAxes(int channel);
	ProbeID QueryProbeStatus(int channel);
	ProbeData ReadData(int channel);
	DisplayData ReadDisplay();

private:
	ViSession VISAresource;
	std::string SendCommand(std::string command, bool query = false, unsigned int waitAfterSend = 50, unsigned int BytesToRead = 100);
};

class FieldAnalyzer
{
public:
	int Initialize(std::string VisaResourceName,
		bool reset = true);
	void Reset();
	int Close();
	std::string QueryID();
	void ConfigureAmplitudeScale(double scale, Units units = Vm);
	void ConfigureTimeBase(int timebase);
	void ConfigureTriggerMode(TriggerMode mode);
	void ConfigureTriggerThreshold(double level);
	void ConfigureTriggerEdge(TriggerEdge edge);
	void ConfigureOffsetFromTrigger(double microseconds);
	void ConfigureUnits(Units units);
	void ConfigureIPAddressMode(AddressMode mode);
	void ConfigureStaticIPAddress(std::string address, std::string subnetMask = "255.255.000.000");
	ProbeID QueryProbeID();
	Status QueryStatus();
	int QueryAmplitudeScale();
	int QueryTimeBase();
	TriggerMode QueryTriggerMode();
	double QueryTriggerThreshold();
	TriggerEdge QueryTriggerEdge();
	double QueryOffsetFromTrigger();
	Units QueryUnits();
	AddressInfo QueryIPAddressMode();
	StaticIP QueryStaticIPAddress();
	double ReadCompositeField();
	MMAData ReadMMA();


private:
	ViSession VISAresource;
	std::string SendCommand(std::string command, bool query = false, unsigned int waitAfterSend = 50, unsigned int BytesToRead = 100);
};